==> jei-1.20.1-fabric-15.20.0.118-sources.jar: mezz/jei/library/gui/widgets/ScrollBoxRecipeWidget.java <==
package mezz.jei.library.gui.widgets;

import mezz.jei.api.gui.drawable.IDrawable;
import mezz.jei.api.gui.inputs.IJeiInputHandler;
import mezz.jei.api.gui.widgets.IScrollBoxWidget;
import mezz.jei.common.Internal;
import mezz.jei.common.config.IClientConfig;
import mezz.jei.common.config.IJeiClientConfigs;
import mezz.jei.common.gui.elements.DrawableBlank;
import mezz.jei.common.gui.elements.DrawableWrappedText;
import mezz.jei.common.util.ImmutableRect2i;
import mezz.jei.common.util.MathUtil;
import net.minecraft.class_332;
import net.minecraft.class_4587;
import net.minecraft.class_5348;
import net.minecraft.class_8030;
import org.joml.Matrix4f;

import java.util.List;

public class ScrollBoxRecipeWidget extends AbstractScrollWidget implements IScrollBoxWidget, IJeiInputHandler {
	private IDrawable contents = DrawableBlank.EMPTY;

	public ScrollBoxRecipeWidget(int width, int height, int xPos, int yPos) {
		super(new ImmutableRect2i(xPos, yPos, width, height));
	}

	@Override
	public int getContentAreaWidth() {
		return contentsArea.width();
	}

	@Override
	public int getContentAreaHeight() {
		return contentsArea.height();
	}

	@Override
	public IScrollBoxWidget setContents(IDrawable contents) {
		this.contents = contents;
		return this;
	}

	@Override
	public IScrollBoxWidget setContents(List<class_5348> text) {
		this.contents = new DrawableWrappedText(text, getContentAreaWidth());
		return this;
	}

	@Override
	protected int getVisibleAmount() {
		return contentsArea.height();
	}

	@Override
	protected int getHiddenAmount() {
		return Math.max(contents.getHeight() - contentsArea.height(), 0);
	}

	@Override
	protected void drawContents(class_332 guiGraphics, double mouseX, double mouseY, float scrollOffsetY) {
		class_4587 poseStack = guiGraphics.method_51448();
		class_4587.class_4665 last = poseStack.method_23760();
		Matrix4f pose = last.method_23761();

		class_8030 scissorArea = MathUtil.transform(contentsArea, pose);
		guiGraphics.method_44379(
			scissorArea.method_49620(),
			scissorArea.method_49618(),
			scissorArea.method_49621(),
			scissorArea.method_49619()
		);
		poseStack.method_22903();
		float scrollAmount = getHiddenAmount() * scrollOffsetY;
		poseStack.method_22904(0.0, -scrollAmount, 0.0);
		try {
			contents.draw(guiGraphics);
		} finally {
			poseStack.method_22909();
			guiGraphics.method_44380();
		}
	}

	@Override
	protected float calculateScrollAmount(double scrollDeltaY) {
		IJeiClientConfigs jeiClientConfigs = Internal.getJeiClientConfigs();
		IClientConfig clientConfig = jeiClientConfigs.getClientConfig();
		int smoothScrollRate = clientConfig.getSmoothScrollRate();

		int totalHeight = contents.getHeight();
		double scrollAmount = scrollDeltaY * smoothScrollRate;
		return (float) (scrollAmount / (double) totalHeight);
	}
}

==> jei-1.21.1-fabric-19.16.5.190-sources.jar: mezz/jei/library/gui/widgets/ScrollBoxRecipeWidget.java <==
package mezz.jei.library.gui.widgets;

import mezz.jei.api.gui.drawable.IDrawable;
import mezz.jei.api.gui.inputs.IJeiInputHandler;
import mezz.jei.api.gui.widgets.IScrollBoxWidget;
import mezz.jei.common.Internal;
import mezz.jei.common.config.IClientConfig;
import mezz.jei.common.config.IJeiClientConfigs;
import mezz.jei.common.util.MathUtil;
import net.minecraft.class_332;
import net.minecraft.class_4587;
import net.minecraft.class_8030;
import org.joml.Matrix4f;

public class ScrollBoxRecipeWidget extends AbstractScrollWidget implements IScrollBoxWidget, IJeiInputHandler {
	private final int visibleHeight;
	private final int hiddenHeight;
	private final class_8030 contentsArea;
	private final IDrawable contents;

	public ScrollBoxRecipeWidget(IDrawable contents, int visibleHeight, int xPos, int yPos) {
		super(new class_8030(
			xPos,
			yPos,
			contents.getWidth() + AbstractScrollWidget.getScrollBoxScrollbarExtraWidth(),
			visibleHeight
		));
		this.contents = contents;
		this.visibleHeight = visibleHeight;
		this.contentsArea = new class_8030(
			0,
			0,
			contents.getWidth(),
			visibleHeight
		);
		this.hiddenHeight = Math.max(contents.getHeight() - visibleHeight, 0);
	}

	@Override
	protected int getVisibleAmount() {
		return visibleHeight;
	}

	@Override
	protected int getHiddenAmount() {
		return hiddenHeight;
	}

	@Override
	protected void drawContents(class_332 guiGraphics, double mouseX, double mouseY, float scrollOffsetY) {
		class_4587 poseStack = guiGraphics.method_51448();
		class_4587.class_4665 last = poseStack.method_23760();
		Matrix4f pose = last.method_23761();

		class_8030 scissorArea = MathUtil.transform(contentsArea, pose);
		guiGraphics.method_44379(
			scissorArea.method_49620(),
			scissorArea.method_49618(),
			scissorArea.method_49621(),
			scissorArea.method_49619()
		);
		poseStack.method_22903();
		float scrollAmount = hiddenHeight * scrollOffsetY;
		poseStack.method_22904(0.0, -scrollAmount, 0.0);
		try {
			contents.draw(guiGraphics);
		} finally {
			poseStack.method_22909();
			guiGraphics.method_44380();
		}
	}

	@Override
	protected float calculateScrollAmount(double scrollDeltaY) {
		IJeiClientConfigs jeiClientConfigs = Internal.getJeiClientConfigs();
		IClientConfig clientConfig = jeiClientConfigs.getClientConfig();
		int smoothScrollRate = clientConfig.getSmoothScrollRate();

		int totalHeight = contents.getHeight();
		double scrollAmount = scrollDeltaY * smoothScrollRate;
		return (float) (scrollAmount / (double) totalHeight);
	}
}
